import 'dart:async';

import 'package:bloc/bloc.dart';
import 'package:flutter/widgets.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:get_it/get_it.dart';
import 'package:unify_api/unify.dart';

import '../../services/database/database.dart';
import '../alerts/alerts_bloc.dart';
import '../alerts/alerts_event.dart';
import '../bloc.dart';
import '../videos/bloc.dart';
import 'bloc.dart';

class AuthBloc extends Bloc<AuthEvent, AuthState> {
  @override
  AuthState get initialState => LoggedOutState();

  @override
  Stream<AuthState> mapEventToState(
    AuthEvent event,
  ) async* {
    final db = GetIt.instance<DatabaseService>();
    if (event is CheckUser) {
      yield* _mapCheckToState(db);
    }
    if (event is LoginEvent) {
      yield* _mapLoginToState(event, db);
    }
    if (event is LogoutEvent) {
      yield* _mapLogoutToState(event, db);
    }
    if (event is SwitchUser) {
      yield* _mapSwitchToState(event, db);
    }
  }

  Stream<AuthState> _mapCheckToState(DatabaseService db) async* {
    final _user = await db.getLastUser();
    yield* _autoLogin(db, _user);
  }

  Stream<AuthState> _mapLoginToState(
      LoginEvent event, DatabaseService db) async* {
    var _domain = event.companyCode;
    if (!_domain.contains('mobile.') && !_domain.contains('api.')) {
      _domain = 'api.$_domain';
    }
    final _client =
        UnifyClient(event.username, event.password, domain: _domain);
    yield* _login(_client, _domain, db, false,
        addAccount: event.addAccount, nav: event.navBloc);
  }

  Stream<AuthState> _mapLogoutToState(
      LogoutEvent event, DatabaseService db) async* {
    yield LogInLoadingState(false);
    await event.user.client.logout();
    yield LoggedOutState();
    final _user = await db.getLastUser();
    await db.deleteUser(_user);
  }

  Stream<AuthState> _mapSwitchToState(
      SwitchUser event, DatabaseService db) async* {
    final _user = event.user;
    final _client =
        UnifyClient(_user.username, _user.password, domain: _user.domain);
    AccountInfo _info;
    await _client.login();
    if (_client != null && _client.isAuthorized) {
      _info = await _client.getInfo();
      if (_info != null) {
        await db.insertUser(_client, _info, _user.domain);
        yield LoggedInState(_user, info: _info);
      }
    }
  }

  Stream<AuthState> _autoLogin(DatabaseService db, User _user) async* {
    if (_user != null && _user?.userID != null) {
      final _client =
          UnifyClient(_user.username, _user.password, domain: _user.domain);
      yield* _login(_client, _user.domain, db, true);
    }
  }

  Stream<AuthState> _login(
      UnifyClient client, String _domain, DatabaseService db, bool auto,
      {NavBloc nav, bool addAccount = false}) async* {
    final _client = client;
    yield LogInLoadingState(auto);
    try {
      await _client.login();
      print('Token -> ${_client.token.accessToken}');
    } catch (e) {
      yield LogInErrorState(e.toString());
    }
    AccountInfo _info;
    if (_client != null && _client.isAuthorized) {
      _info = await _client.getInfo();
      if (_info != null) {
        User _user;
        try {
          _user = await db.insertUser(_client, _info, _domain);
        } catch (e) {
          print('Error Inser User: $e');
        }
        yield LoggedInState(_user, info: _info);
      } else {
        yield LogInErrorState('User Info Not Found!');
      }
    } else {
      yield LogInErrorState('Username or Password Incorrect!');
    }
  }

  static User currentUser(BuildContext context) {
    final auth = BlocProvider.of<AuthBloc>(context);
    final state = auth.currentState;
    if (state is LoggedInState) {
      return state.user;
    }
    return null;
  }

  static AccountInfo currentInfo(BuildContext context) {
    final auth = BlocProvider.of<AuthBloc>(context);
    final state = auth.currentState;
    if (state is LoggedInState) {
      return state.info;
    }
    return null;
  }

  static void refresh(BuildContext context) {
    final user = AuthBloc.currentUser(context);
    final info = AuthBloc.currentInfo(context);
    final scaffold = BlocProvider.of<ScaffoldBloc>(context)
      ..dispatch(ShowSnakBar('Welcome Back ${user.firstName}!', context));
    BlocProvider.of<LoansBloc>(context)
        .dispatch(LoansCheck(user: info, scaffold: scaffold));
    BlocProvider.of<ContactsBloc>(context)
        .dispatch(ContactsCheck(user: info, scaffold: scaffold));
    BlocProvider.of<LeadsBloc>(context)
        .dispatch(LeadsCheck(user: info, scaffold: scaffold));
    BlocProvider.of<AlertsBloc>(context)
        .dispatch(AlertsCheck(user: info, scaffold: scaffold));
    BlocProvider.of<VideosBloc>(context)
        .dispatch(VideosCheck(user: info, scaffold: scaffold));
  }
}
