import 'package:flutter/foundation.dart';
import 'package:moor/moor.dart';
import 'package:unify_api/unify.dart' as api;

import '../database.dart';
import '../tables/loan.dart';

part 'loan.g.dart';

@UseDao(
  tables: [LoanRecords],
  queries: {
    'loansById':
        'SELECT * FROM loans WHERE user_i_d = :userID AND loan_id = :loanID LIMIT 1',
    'loansByFolder':
        'SELECT * FROM loans WHERE user_i_d = :userID AND folder = :folder ORDER BY last_name ASC',
  },
)
class LoansDao extends DatabaseAccessor<DatabaseService> with _$LoansDaoMixin {
  LoansDao(DatabaseService db) : super(db);

  Future batchUpdate(
    String userID,
    List<api.Loan> loans, {
    api.LoansFolder folder = api.LoansFolder.all,
  }) async {
    final _items = <LoanRecordsCompanion>[];
    final _updated = DateTime.now();
    for (var loan in loans) {
      final _item = LoanRecordsCompanion(
        loanId: Value(loan.id),
        folder: addField<String>(describeEnum(folder),
            fallback: describeEnum(api.LoansFolder.all)),
        userID: Value(userID),
        firstName: addField<String>(loan?.firstName),
        lastName: addField<String>(loan?.lastName),
        cellPhone: addField<String>(phoneFormat(loan?.cellPhone)),
        homePhone: addField<String>(phoneFormat(loan?.homePhone)),
        officePhone: addField<String>(phoneFormat(loan?.homePhone)),
        email: addField<String>(loan?.email),
        dateClosed: addField<DateTime>(loan?.dateClosed),
        loanMilestone: addField<String>(loan?.loanMilestone),
        loanNumber: addField<String>(loan?.loanNumber),
        loanRate: addField<double>(loan?.loanRate),
        lastUpdated: addField<DateTime>(_updated),
      );
      _items.add(_item);
    }
    await into(loanRecords).insertAll(_items, orReplace: true);
    await cleanLoans(userID, _updated, folder: folder);
  }

  Future<List<Loan>> searchLoans(
    String userID,
    String query, {
    api.LoansFolder folder = api.LoansFolder.all,
    LoanScope filter,
  }) {
    final scope = filter ?? LoanScope();
    final sb = StringBuffer(
        'SELECT * FROM loans WHERE user_i_d = :userID AND folder = :folder AND (');
    bool _first = true;
    if (scope.email) {
      if (!_first) sb.write(' OR ');
      sb.writeln('email like :query');
      _first = false;
    }
    if (scope.firstName) {
      if (!_first) sb.write(' OR ');
      sb.writeln('first_name like :query');
      _first = false;
    }
    if (scope.lastName) {
      if (!_first) sb.write(' OR ');
      sb.writeln('last_name like :query');
      _first = false;
    }
    if (scope.loanMilestone) {
      if (!_first) sb.write(' OR ');
      sb.writeln('loan_milestone like :query');
      _first = false;
    }
    if (scope.loanRate) {
      if (!_first) sb.write(' OR ');
      sb.writeln('loan_rate like :query');
      _first = false;
    }
    if (scope.loanNumber) {
      if (!_first) sb.write(' OR ');
      sb.writeln('loan_number like :query');
      _first = false;
    }
    if (scope.cellPhone) {
      if (!_first) sb.write(' OR ');
      sb.writeln('cell_phone like :query');
      _first = false;
    }
    if (scope.homePhone) {
      if (!_first) sb.write(' OR ');
      sb.writeln('home_phone like :query');
      _first = false;
    }
    if (scope.officePhone) {
      if (!_first) sb.write(' OR ');
      sb.writeln('office_phone like :query');
      _first = false;
    }
    sb.writeln(')');
    final _search = query
        .replaceAll('-', '')
        .replaceAll(' ', '')
        .replaceAll('(', '')
        .replaceAll(')', '')
        .replaceAll(';', '');
    print('SQL -> ${sb.toString()}');
    return customSelectQuery(
      sb.toString(),
      variables: [
        Variable.withString(userID),
        Variable.withString(describeEnum(folder)),
        Variable.withString('%$_search%'),
      ],
      readsFrom: {loanRecords},
    ).map((row) => Loan.fromData(row.data, db)).get();
    // return (select(loanRecords)
    //       ..where((l) => l.userID.equals(userID))
    //       ..where((l) => l.folder.equals(describeEnum(folder)))
    //       ..where((l) {
    //         final _scopes = <Expression<bool, BoolType>>[];
    //         if (scope.email) {
    //           _scopes.add(l.email.like('%$query%'));
    //         }
    //         if (scope.firstName) {
    //           _scopes.add(l.firstName.like('%$query%'));
    //         }
    //         if (scope.lastName) {
    //           _scopes.add(l.lastName.like('%$query%'));
    //         }
    //         if (scope.loanMilestone) {
    //           _scopes.add(l.loanMilestone.like('%$query%'));
    //         }
    //         if (scope.loanNumber) {
    //           _scopes.add(l.loanNumber.like('%$query%'));
    //         }
    //         if (num.tryParse(query) != null) {
    //           final number = num.tryParse(query);
    //           if (scope.loanRate) {
    //             _scopes.add(l.loanRate.equals(number));
    //           }
    //         }
    //         final _num = query
    //             .replaceAll('-', '')
    //             .replaceAll(' ', '')
    //             .replaceAll('(', '')
    //             .replaceAll(')', '')
    //             .replaceAll(';', '');
    //         if (scope.cellPhone) {
    //           _scopes.add(l.cellPhone.like('%$_num%'));
    //         }
    //         if (scope.homePhone) {
    //           _scopes.add(l.homePhone.like('%$_num%'));
    //         }
    //         if (scope.officePhone) {
    //           _scopes.add(l.officePhone.like('%$_num%'));
    //         }

    //         Expression<bool, BoolType> _exp, _a, _b;
    //         while (_scopes.isNotEmpty) {
    //           _a = _scopes.removeAt(0);
    //           _b = _scopes.length > 1 ? _scopes.removeAt(1) : null;
    //           _exp = ExpSearch(a: _a, b: _b, c: _exp).exp;
    //         }

    //         return _exp;
    //       }))
    //     .get();
  }

  Future<Loan> getLastLoan(String userID, api.LoansFolder folder) async {
    final _items = await (select(loanRecords)
          ..where((l) => l.userID.equals(userID))
          ..where((l) => l.folder.equals(describeEnum(folder)))
          ..orderBy([
            (u) => OrderingTerm(
                expression: u.lastUpdated, mode: OrderingMode.desc),
          ]))
        .get();
    if (_items != null && _items.isNotEmpty) {
      return _items.first;
    }
    return null;
  }

  Future<Loan> getLoan(String userID, String loanID) async {
    final _loans = await loansById(userID, loanID);
    if (_loans != null && _loans.isNotEmpty) {
      return _loans.first;
    }
    return null;
  }

  Future resetLoansForUser(String userID) {
    return (delete(loanRecords)..where((t) => t.userID.equals(userID))).go();
  }

  Future cleanLoans(
    String userID,
    DateTime lastUpdated, {
    api.LoansFolder folder = api.LoansFolder.all,
  }) {
    return (delete(loanRecords)
          ..where((t) => t.userID.equals(userID))
          ..where((t) => t.folder.equals(describeEnum(folder)))
          ..where((l) => l.lastUpdated.isBiggerThanValue(lastUpdated)))
        .go();
  }

  Future insertDetails(String userID, String loanID, String details) async {
    final loan = await loansById(userID, loanID);

    if (loan != null && loan.isNotEmpty) {
      final _loan = loan.first
          .copyWith(loanDetails: details, lastUpdated: DateTime.now());
      return into(loanRecords).insert(_loan, orReplace: true);
    }
  }

  /// Get a Phone Number from a 10 digit string (after symbols removed)
  String phoneFormat(String value) {
    String areaCode, prefix, number, ext;
    var _number = value;
    if (_number.length >= 10) {
      areaCode = _number.substring(0, 3);
      prefix = _number.substring(3, 6);
      number = _number.substring(6, 10);
      if (_number.length > 10) {
        ext = value.substring(10, _number.length);
      }
      if (_number.isEmpty) return "";
      if (ext != null) {
        // print('($areaCode) $prefix-$number :$ext');
        return '($areaCode) $prefix-$number :$ext'.toString();
      }
      // print('($areaCode) $prefix-$number');
      return '($areaCode) $prefix-$number'.toString();
    }
    return '';
  }
}

class LoanScope {
  LoanScope({
    this.email = true,
    this.cellPhone = true,
    this.homePhone = true,
    this.officePhone = true,
    this.firstName = true,
    this.lastName = true,
    this.loanMilestone = true,
    this.loanNumber = true,
    this.loanRate = false,
  });
  bool email,
      cellPhone,
      homePhone,
      officePhone,
      firstName,
      lastName,
      loanMilestone,
      loanRate,
      loanNumber;
}
