import 'package:data_tables/ui/stateless_datatable.dart';
import 'package:flutter/material.dart';
import 'package:get_it/get_it.dart';
import 'package:intl/intl.dart';

import '../../../data/blocs/bloc.dart';
import '../../../data/services/database/database.dart';

class ContactsDataTable extends StatefulWidget {
  @override
  _ContactsDataTableState createState() => _ContactsDataTableState();
}

class _ContactsDataTableState extends State<ContactsDataTable> {
  final _controller = TextEditingController();
  int _firstRowIndex = 0;
  String _query = '';
  int _rowsPerPage = PaginatedDataTable.defaultRowsPerPage;
  final _selected = <Contact>[];
  bool _sortAscending = true;
  int _sortColumnIndex = 1;
  final _sortFields = [
    SortField.firstName,
    SortField.lastName,
    SortField.email,
    SortField.cellPhone,
    SortField.homePhone,
    SortField.officePhone,
  ];

  @override
  void initState() {
    super.initState();
  }

  SortField get currentField => _sortFields[_sortColumnIndex];

  void _sort(int columnIndex, bool ascending) {
    setState(() {
      _sortColumnIndex = columnIndex;
      _sortAscending = ascending;
    });
  }

  Row _buildHeader() {
    return Row(
      children: <Widget>[
        Expanded(
            child: TextField(
          controller: _controller,
          decoration: InputDecoration(
              hintText: 'Search..',
              filled: true,
              border: InputBorder.none,
              prefixIcon: Icon(Icons.search),
              suffixIcon: IconButton(
                padding: EdgeInsets.all(0.0),
                icon: Icon(Icons.clear),
                onPressed: () {
                  _controller.clear();
                  _onSearchChanged('');
                },
              )),
          onChanged: _onSearchChanged,
        )),
      ],
    );
  }

  void _onSearchChanged(String val) {
    if (mounted)
      setState(() {
        _query = val;
      });
  }

  @override
  Widget build(BuildContext context) {
    final _user = AuthBloc.currentUser(context);
    final _dao = GetIt.instance<DatabaseService>().contactsDao;
    return Scaffold(
      body: FutureBuilder<List<Contact>>(
          future: _query != null && _query.isNotEmpty
              ? _dao.searchContacts(_user.userID, _query)
              : _dao.getContacts(_user.userID, currentField, _sortAscending),
          builder: (context, snapshot) {
            final _contacts = snapshot.data;
            return StatelessDataTable(
              header: _buildHeader(),
              rowsPerPage: _rowsPerPage,
              onRowsPerPageChanged: (value) {
                setState(() {
                  _rowsPerPage = value;
                });
              },
              firstRowIndex: _firstRowIndex,
              handleNext: () {
                if (mounted)
                  setState(() {
                    _firstRowIndex += _rowsPerPage;
                  });
              },
              handlePrevious: () {
                if (mounted)
                  setState(() {
                    _firstRowIndex -= _rowsPerPage;
                  });
              },
              onPageChanged: (val) {},
              sortColumnIndex: _sortColumnIndex,
              sortAscending: _sortAscending,
              onSelectAll: (val) {
                if (mounted)
                  setState(() {
                    if (val) {
                      _selected.addAll(_contacts);
                    } else {
                      _selected.clear();
                    }
                  });
              },
              columns: [
                DataColumn(
                  label: const Text('First Name'),
                  onSort: _sort,
                ),
                DataColumn(
                  label: const Text('Last Name'),
                  onSort: _sort,
                ),
                DataColumn(
                  label: const Text('Email'),
                  onSort: _sort,
                ),
                DataColumn(
                  label: const Text('Cell Phone'),
                  onSort: _sort,
                ),
                DataColumn(
                  label: const Text('Home Phone'),
                  onSort: _sort,
                ),
                DataColumn(
                  label: const Text('Office Phone'),
                  onSort: _sort,
                ),
                DataColumn(
                  label: const Text('Last Activity'),
                  onSort: _sort,
                ),
                DataColumn(
                  label: const Text('Date Created'),
                  onSort: _sort,
                ),
              ],
              rows: <DataRow>[
                if (_contacts != null)
                  for (var item in _contacts) ...[
                    DataRow.byIndex(
                      index: _contacts.indexOf(item),
                      selected: _selected.contains(item),
                      onSelectChanged: (value) {
                        if (mounted)
                          setState(() {
                            if (value) {
                              _selected.add(item);
                            } else {
                              _selected.remove(item);
                            }
                          });
                      },
                      cells: [
                        DataCell(
                          Text('${item.firstName}'),
                        ),
                        DataCell(
                          Text('${item.lastName}'),
                        ),
                        DataCell(
                          Text('${item.email}'),
                        ),
                        DataCell(
                          Text('${item.cellPhone}'),
                        ),
                        DataCell(
                          Text('${item.homePhone}'),
                        ),
                        DataCell(
                          Text('${item.officePhone}'),
                        ),
                        DataCell(
                          Text('${item.lastActivity}'),
                        ),
                        DataCell(
                          Text(DateFormat.yMd().format(item.dateCreated)),
                        ),
                      ],
                    )
                  ],
              ],
            );
          }),
    );
  }
}
