import 'package:fab_menu/fab_menu.dart';
import 'package:flutter/material.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:get_it/get_it.dart';

import '../../../data/blocs/bloc.dart';
import '../../../data/classes/enum_list.dart';
import '../../../data/services/database/dao/contact.dart';
import '../../../data/services/database/database.dart';
import '../../../data/utils/navigate.dart';
import 'add_contact.dart';
import 'index.dart';
import 'tile.dart';

class ContactsListScreen extends StatefulWidget {
  @override
  _ContactsListScreenState createState() => _ContactsListScreenState();

  static refresh(BuildContext context) {
    final contacts = BlocProvider.of<ContactsBloc>(context);
    final _user = AuthBloc.currentInfo(context);
    final scaffold = BlocProvider.of<ScaffoldBloc>(context);
    return contacts.dispatch(
        RefreshContacts(user: _user, scaffold: scaffold, context: context));
  }
}

class _ContactsListScreenState extends State<ContactsListScreen> {
  String _query;
  ContactScope _scope = ContactScope();
  List<MenuData> menuDataList;
  final _controller = TextEditingController();
  @override
  Widget build(BuildContext context) {
    final _user = AuthBloc.currentUser(context);

    int favId;

    final db = GetIt.instance<DatabaseService>();
    return LayoutBuilder(
      builder: (context, dimens) {
        return Scaffold(
          floatingActionButton: FloatingActionButton(
            heroTag: 'contact_add',
            // mini: true,
            backgroundColor: Theme.of(context).accentColor,
            child: Icon(Icons.add),
            onPressed: () {
              navigate(context, AddEditContact(), fullScreen: true);
            },
          ),
          body: BlocBuilder<ContactsBloc, ContactsState>(
            builder: (context, state) {
              return Column(
                children: <Widget>[
                  if (state is ContactsLoadingState) LinearProgressIndicator(),
                  Card(
                    elevation: 8.0,
                    child: TextField(
                      controller: _controller,
                      decoration: InputDecoration(
                          filled: true,
                          prefixIcon: IconButton(
                            icon: Icon(Icons.filter_list),
                            onPressed: () {
                              showDialog<ContactScope>(
                                context: context,
                                builder: (_) => AlertDialog(
                                  title: Text('Filter'),
                                  content: SingleChildScrollView(
                                    child: ContactFilterPopup(
                                      filters: _scope,
                                      onChanged: (val) {
                                        if (mounted)
                                          setState(() {
                                            _scope = val;
                                          });
                                      },
                                    ),
                                  ),
                                ),
                              );
                            },
                          ),
                          border: InputBorder.none,
                          hintText: 'Search Contacts..',
                          suffixIcon: IconButton(
                            icon: Icon(Icons.clear),
                            onPressed: () {
                              _controller.clear();
                              if (mounted)
                                setState(() {
                                  _query = null;
                                });
                            },
                          )),
                      onChanged: (val) {
                        if (mounted)
                          setState(() {
                            _query = val;
                          });
                      },
                    ),
                  ),
                  Expanded(
                    child: Container(
                      child: StreamBuilder<List<Contact>>(
                          stream: _getStream(db, _user),
                          builder: (context, snapshot) {
                            if (snapshot.hasData) {
                              final _contacts = snapshot.data;
                              if (_contacts.isEmpty) {
                                if (state is LoansLoadingState) {
                                  return Column(
                                    mainAxisAlignment: MainAxisAlignment.center,
                                    crossAxisAlignment:
                                        CrossAxisAlignment.center,
                                    children: [
                                      Text('No Contacts Found'),
                                      SizedBox(height: 20.0),
                                      CircularProgressIndicator(),
                                    ],
                                  );
                                }
                                return Center(child: Text('No Contacts Found'));
                              }
                              return Scrollbar(
                                child: RefreshIndicator(
                                  child: ListView.separated(
                                    separatorBuilder: (context, index) =>
                                        Divider(height: 1),
                                    itemCount: _contacts.length,
                                    itemBuilder: (context, index) {
                                      final _contact = _contacts[index];
                                      return Padding(
                                        padding: const EdgeInsets.all(7.0),
                                        child: StreamBuilder<List<Favorite>>(
                                            stream: GetIt.instance<
                                                    DatabaseService>()
                                                .favoritesDao
                                                .watchFavoriteByTypeId(
                                                    _user.userID,
                                                    _contact.contactId),
                                            builder: (context, snapshot) {
                                              if (snapshot.hasData &&
                                                  snapshot.data.isNotEmpty) {
                                                favId = snapshot
                                                    .data.first.favoriteId;
                                              }
                                              return ContactRowTile(
                                                favorite: snapshot.hasData &&
                                                    snapshot.data.isNotEmpty,
                                                contact: _contact,
                                                type: ViewType.detailed,
                                                favId: favId,
                                              );
                                            }),
                                      );
                                    },
                                  ),
                                  onRefresh: () async {
                                    ContactsListScreen.refresh(context);
                                    await Future.delayed(Duration(seconds: 3));
                                  },
                                ),
                              );
                            }
                            return Center(child: CircularProgressIndicator());
                          }),
                    ),
                  ),
                ],
              );
            },
          ),
        );
      },
    );
  }

  Stream<List<Contact>> _getStream(DatabaseService db, User _user) {
    if (_query == null || _query.isEmpty) {
      return db.contactsDao.watchContactsByUser(_user.userID);
    }
    return db.contactsDao
        .searchContacts(_user.userID, _query, filter: _scope)
        .asStream();
  }
}
