import 'dart:convert';

import 'package:flip_card/flip_card.dart';
import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';
import 'package:flutter/rendering.dart';
import 'package:flutter_sparkline/flutter_sparkline.dart';
import 'package:get_it/get_it.dart';
import 'package:hive/hive.dart';
import 'package:http/http.dart' as http;

import '../../../data/blocs/auth/bloc.dart';
import '../../../data/classes/zillow_graph.dart';
import '../../../data/services/database/database.dart';
import '../../common/raised_card.dart';

class CurrentRatesCard extends StatefulWidget {
  static const String cardKey = 'card_rates';
  static const BoxConstraints constraints = BoxConstraints(
    minHeight: 50.0,
    minWidth: 50.0,
    maxHeight: double.infinity,
    maxWidth: 400.0,
  );
  final int id;
  final bool preview;

  const CurrentRatesCard({
    Key key,
    @required this.id,
    this.preview = false,
  }) : super(key: key);

  @override
  _CurrentRatesCardState createState() => _CurrentRatesCardState();
}

class _CurrentRatesCardState extends State<CurrentRatesCard> {
  Box box;
  List<double> ratesList = [];
  final _dropItems = ['7 Days', '30 Days', '1 Year'];
  double _currentRate = 0.0;
  String actualDropdown;
  bool noNetwork = false;
  GlobalKey<FlipCardState> cardKey = GlobalKey<FlipCardState>();

  @override
  void initState() {
    init();
    super.initState();
  }

  Future init() async {
    box = await Hive.openBox('app_rates_${widget?.id ?? 0}');
    actualDropdown = await box.get('currentDropdown') ?? _dropItems.first;
    _loadRates(actualDropdown);
  }

  @override
  Widget build(BuildContext context) {
    final _user = AuthBloc.currentUser(context);
    return GestureDetector(
      onDoubleTap: () => {
        if(!widget.preview)
          cardKey.currentState.toggleCard()
      },
      child: FlipCard(
        key: cardKey,
        speed: 450,
        flipOnTouch: false,
        front: RaisedChild(
          Container(
            decoration: BoxDecoration(borderRadius: BorderRadius.circular(25)),
            child: Column(
              children: <Widget>[
                Padding(
                  padding: const EdgeInsets.all(8.0),
                  child: Row(
                    mainAxisAlignment: MainAxisAlignment.spaceBetween,
                    crossAxisAlignment: CrossAxisAlignment.start,
                    children: <Widget>[
                      Column(
                        mainAxisAlignment: MainAxisAlignment.start,
                        crossAxisAlignment: CrossAxisAlignment.start,
                        children: <Widget>[
                          Text('Current Rate: '),
                          Builder(builder: (_) {
                            if (_currentRate != null) {
                              return Text('$_currentRate',
                                  style: Theme.of(context).textTheme.title);
                            } else {
                              return CircularProgressIndicator();
                            }
                          })
                        ],
                      ),
                      Container(
                        child: DropdownButton(
                            isDense: true,
                            value: actualDropdown ?? _dropItems?.first,
                            onChanged: (value) => setState(() {
                                  if (mounted)
                                    setState(() {
                                      actualDropdown = value;
                                    });
                                  _loadRates(actualDropdown);
                                }),
                            items: _dropItems.map((title) {
                              return DropdownMenuItem(
                                value: title,
                                child: Text(title,
                                    style: TextStyle(
                                        color: Colors.blue,
                                        fontWeight: FontWeight.w400,
                                        fontSize: 14.0)),
                              );
                            }).toList()),
                      )
                    ],
                  ),
                ),
                Padding(
                  padding: const EdgeInsets.all(10.0),
                  child: Builder(
                    builder: (_) {
                      if (ratesList == null || ratesList.isEmpty) {
                        if (noNetwork == true) {
                          return Container(
                            child: Center(
                                child:
                                    Column(
                                      children: <Widget>[
                                        Text('Please check Internet connection'),
                                        FlatButton.icon(
                                          icon: Icon(Icons.refresh),
                                          label: Text('Refresh Rates'),
                                          onPressed: () => _loadRates(actualDropdown),
                                        )
                                      ],
                                    )),
                          );
                        }
                        return Center(
                          child: CircularProgressIndicator(),
                        );
                      }
                      return InkWell(
                        onTap: () => _loadRates(actualDropdown),
                        child: Sparkline(
                          data: ratesList.cast<double>(),
                          lineColor: Colors.green,
                          lineWidth: 4,
                        ),
                      );
                    },
                  ),
                ),
              ],
            ),
          ),
        ),
        back: RaisedChild(
          Container(
            height: 175,
            width: 400,
            decoration: BoxDecoration(borderRadius: BorderRadius.circular(25)),
            child: Column(
              mainAxisAlignment: MainAxisAlignment.center,
              crossAxisAlignment: CrossAxisAlignment.center,
              children: <Widget>[
                FlatButton.icon(
                  label: Text('Delete Widget'),
                  icon: Icon(Icons.delete),
                  onPressed: () {
                    deleteWidget(_user, widget.id);
                  },
                )
              ],
            ),
          ),
        ),
      ),
    );
  }

  void deleteWidget(User user, int id) async {
    await GetIt.instance<DatabaseService>().homeDao.removeItem(id, user.userID);
  }

  void _loadRates(String length) async {
    //await storage.init();
    var _response;
    String key;
    try {
      if (length == '7 Days') {
        _response = await http.get(
            'https://mortgageapi.zillow.com/getRates?partnerId=RD-GBPRNSZ&durationDays=7&includeCurrentRate=true');
        key = '7 Days';
      } else if (length == '30 Days') {
        _response = await http.get(
            'https://mortgageapi.zillow.com/getRates?partnerId=RD-GBPRNSZ&durationDays=30&includeCurrentRate=true');
        key = '30 Days';
      } else if (length == '1 Year') {
        _response = await http.get(
            'https://mortgageapi.zillow.com/getRates?partnerId=RD-GBPRNSZ&durationDays=365&includeCurrentRate=true');
        key = '1 Year';
      }

      if (_response.statusCode == 200) {
        final _graph = ZillowGraph.fromJson(json.decode(_response.body));
        if (mounted)
          setState(() {
            _currentRate = _graph?.rates?.rates?.currentRate?.rate ?? 1;
            ratesList =
                _graph?.rates?.rates?.samples?.map((s) => s.rate)?.toList() ??
                    [];
          });

        box.put(key, ratesList);
        box.put('currentRate', _currentRate);
        box.put('currentDropdown', actualDropdown);
      }
    } catch (e) {
      print('ERROR calling API: $e');
      if (box.isOpen) {
        if (mounted) {
          setState(() {
            if(box.isEmpty){
              noNetwork = true;
            }
            ratesList = List.from(box.get(actualDropdown))
                .map((i) => double.tryParse(i.toString()))
                .toList();
            _currentRate = box.get('currentRate');
          });
        }
      }
    }
  }
}
