import 'package:flutter/material.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:intl/intl.dart';
import 'package:unify_api/unify.dart' as unify;

import '../../../data/blocs/bloc.dart';
import '../../../data/services/database/database.dart';
import 'common/index.dart';

class StartCampaignScreen extends StatefulWidget {
  const StartCampaignScreen({this.contacts, this.leads, this.material});
  static const String routeName = '/start_campaign';

  final List<Contact> contacts;
  final List<Lead> leads;
  final unify.MarketingMaterial material;
  @override
  _StartCampaignScreenState createState() => _StartCampaignScreenState();
}

class _StartCampaignScreenState extends State<StartCampaignScreen> {
  int _currentStep = 0;
  final _scaffoldKey = GlobalKey<ScaffoldState>();
  unify.MarketingMaterial _material;
  List<Contact> _contacts;
  List<Lead> _leads;
  DateTime _date = DateTime.now();
  bool _sendNow = false;

  @override
  void initState() {
    super.initState();
    if(widget.contacts != null && widget.contacts.isNotEmpty){
      _contacts = widget.contacts;
    }
    if(widget.leads != null && widget.leads.isNotEmpty){
      _leads = widget.leads;
    } 
    if(widget.material != null){
      _material = widget.material;
    }
  }

  @override
  Widget build(BuildContext context) {
    return BlocBuilder<AuthBloc, AuthState>(
      builder: (context, state) {
        final _steps = [
          if (state is LoggedInState) ...[
            Step(
              title: Text('Choose a Campaign'),
              content: Card(
                child: Padding(
                  padding: const EdgeInsets.all(8.0),
                  child: Center(
                    child: CampaignSelect(
                      material: _material,
                      client: state.info.client,
                      onSelect: (val) {
                        if (mounted)
                          setState(() {
                            _material = val;
                          });
                      },
                    ),
                  ),
                ),
              ),
              state: _currentStep == 0
                  ? StepState.editing
                  : _material != null ? StepState.complete : StepState.indexed,
              isActive: true,
            ),
            Step(
              title: Text('Select Contacts and Leads'),
              content: Card(
                child: Padding(
                  padding: const EdgeInsets.all(8.0),
                  child: Center(
                    child: ContactLeadSelect(
                      contacts: _contacts,
                      leads: _leads,
                      contactsChanged: (items) {
                        if (mounted)
                          setState(() {
                            _contacts = items;
                          });
                      },
                      leadsChanged: (items) {
                        if (mounted)
                          setState(() {
                            _leads = items;
                          });
                      },
                    ),
                  ),
                ),
              ),
              state: _currentStep == 1
                  ? StepState.editing
                  : (_contacts != null || _leads != null)
                      ? StepState.complete
                      : StepState.indexed,
              isActive: true,
            ),
            Step(
              title: Text('Pick a Date'),
              content: Card(
                child: Padding(
                  padding: const EdgeInsets.all(8.0),
                  child: Center(
                    child: Row(
                      mainAxisAlignment: MainAxisAlignment.spaceBetween,
                      children: <Widget>[
                        if (!_sendNow) ...[
                          Flexible(
                            flex: 1,
                            child: Row(
                              children: <Widget>[
                                RaisedButton.icon(
                                  icon: Icon(Icons.event),
                                  label: Text(DateFormat.yMd().format(_date)),
                                  onPressed: () async {
                                    final _item = await showDatePicker(
                                      context: context,
                                      firstDate: DateTime.now()
                                          .subtract(Duration(days: 3)),
                                      lastDate: DateTime.now()
                                          .add(Duration(days: 700)),
                                      initialDate: _date,
                                    );
                                    if (_item != null) {
                                      if (mounted)
                                        setState(() {
                                          _date = _item;
                                        });
                                    }
                                  },
                                ),
                              ],
                            ),
                          ),
                        ] else ...[
                          Flexible(
                            flex: 1,
                            child: ListTile(
                              leading: Icon(Icons.event),
                              title: Text(DateFormat.yMd().format(_date)),
                            ),
                          ),
                        ],
                        Spacer(),
                        Flexible(
                          flex: 1,
                          child: SwitchListTile.adaptive(
                            title: Text('Send Now'),
                            value: _sendNow,
                            onChanged: (val) {
                              if (mounted)
                                setState(() {
                                  _sendNow = val;
                                  if (_sendNow) {
                                    _date = DateTime.now();
                                  }
                                });
                            },
                          ),
                        )
                      ],
                    ),
                  ),
                ),
              ),
              isActive: true,
              state: _currentStep == 2
                  ? StepState.editing
                  : _date != null ? StepState.complete : StepState.indexed,
            ),
            Step(
              title: Text('Review'),
              content: Card(child: _buildMarketingReview(state)),
              isActive: true,
              state: _currentStep == 3 ? StepState.editing : StepState.indexed,
            ),
          ],
        ];
        return Scaffold(
          key: _scaffoldKey,
          appBar: AppBar(
            title: Text('Start Marketing Campaign'),
          ),
          body: Stepper(
            currentStep: _currentStep,
            steps: _steps,
            type: StepperType.vertical,
            onStepTapped: _onStepTap,
            onStepCancel: _onStepCancel,
            onStepContinue: state is LoggedInState
                ? () => _onStepContinue(_steps, state.info.client)
                : null,
          ),
        );
      },
    );
  }

  Widget _buildMarketingReview(LoggedInState state) {
    if (_material == null) {
      return Container(
        height: 100,
        child: Center(
          child: RaisedButton(
            child: Text('Pick a Campaign'),
            onPressed: () {
              if (mounted)
                setState(() {
                  _currentStep = 0;
                });
            },
          ),
        ),
      );
    }
    if ((_contacts == null || _contacts.isEmpty) &&
        (_leads == null || _leads.isEmpty)) {
      return Container(
        height: 100,
        child: Center(
          child: RaisedButton(
            child: Text('Pick Contacts/Leads'),
            onPressed: () {
              if (mounted)
                setState(() {
                  _currentStep = 1;
                });
            },
          ),
        ),
      );
    }
    if (_date == null) {
      return Container(
        height: 100,
        child: Center(
          child: RaisedButton(
            child: Text('Pick a Date'),
            onPressed: () {
              if (mounted)
                setState(() {
                  _currentStep = 2;
                });
            },
          ),
        ),
      );
    }
    return Container(
      child: FutureBuilder<unify.MarketingValidation>(
        future: state.info.client.getMarketingValidation(
          _material.id,
          sendNow: _sendNow,
          startDate: _date,
          contactIds: (_contacts == null || _contacts.isEmpty)
              ? []
              : _contacts.map((i) => i.contactId).toList(),
          leadIds: (_leads == null || _leads.isEmpty)
              ? []
              : _leads.map((i) => i.leadId).toList(),
        ),
        builder: (context, snapshot) {
          if (snapshot.hasData) {
            final validation = snapshot.data;
            return Center(
              child: Column(
                children: <Widget>[
                  ListTile(
                    title: Text(
                      _material?.name,
                      style: TextStyle(
                        fontWeight: FontWeight.w600,
                        fontSize: 22.0,
                      ),
                    ),
                  ),
                  ListTile(
                    title: Text(_material?.description),
                  ),
                  Divider(),
                  ListTile(
                    title: Text(
                        'Number of Contacts/Leads selected for marketing:'),
                    trailing: Text(
                      (validation?.selected ?? 0).toString(),
                      style: Theme.of(context).textTheme.display1,
                    ),
                  ),
                  ListTile(
                    title:
                        Text('Number of Contacts/Leads in a licensed state:'),
                    trailing: Text(
                      (validation?.licensed ?? 0).toString(),
                      style: Theme.of(context).textTheme.display1,
                    ),
                  ),
                  ListTile(
                    title: Text(
                        'Number of Contacts/Leads with a valid email address:'),
                    trailing: Text(
                      (validation?.valid ?? 0).toString(),
                      style: Theme.of(context).textTheme.display1,
                    ),
                  ),
                  ListTile(
                    title: Text('Contacts/Leads that have opted out:'),
                    trailing: Text(
                      (validation?.optOut ?? 0).toString(),
                      style: Theme.of(context).textTheme.display1,
                    ),
                  ),
                  ListTile(
                    title: Text('Number of Exclusive Contacts/Leads:'),
                    trailing: Text(
                      (validation?.exclusive ?? 0).toString(),
                      style: Theme.of(context).textTheme.display1,
                    ),
                  ),
                  ListTile(
                    title: Text('Total Campaigns that will be created:'),
                    trailing: Text(
                      (validation?.campaigns ?? 0).toString(),
                      style: Theme.of(context).textTheme.display1,
                    ),
                  ),
                ],
              ),
            );
          }
          return Container(
            height: 100,
            child: Center(
              child: CircularProgressIndicator(),
            ),
          );
        },
      ),
    );
  }

  void _onStepContinue(List<Step> _steps, unify.UnifyClient client) async {
    if (_currentStep < _steps.length - 1) {
      setState(() {
        _currentStep += 1;
      });
    } else {
      try {
        await client.startMarketingCampaign(
          _material.id,
          sendNow: _sendNow,
          startDate: _date,
          contactIds: (_contacts == null || _contacts.isEmpty)
              ? []
              : _contacts.map((i) => i.contactId).toList(),
          leadIds: (_leads == null || _leads.isEmpty)
              ? []
              : _leads.map((i) => i.leadId).toList(),
        );
        _scaffoldKey.currentState.showSnackBar(SnackBar(
          content: Text('Campaigns Started!'),
        ));
        await Future.delayed(Duration(seconds: 3));
        Navigator.pop(context);
      } catch (e) {
        _scaffoldKey.currentState.showSnackBar(SnackBar(
          content: Text(e.toString()),
        ));
      }
    }

    print('onStepContinue : ' + _currentStep.toString());
  }

  void _onStepCancel() {
    setState(() {
      if (_currentStep > 0) {
        _currentStep -= 1;
      } else {
        _currentStep = 0;
      }
    });

    print('onStepCancel : ' + _currentStep.toString());
  }

  void _onStepTap(int step) {
    setState(() {
      _currentStep = step;
    });

    print('onStepTapped : ' + step.toString());
  }
}
