import 'package:flutter/material.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:get_it/get_it.dart';
import 'package:unify_api/unify.dart' as api;

import '../../../data/blocs/auth/bloc.dart';
import '../../../data/services/database/database.dart';
import '../../common/app_input_field.dart';
import 'select_emails.dart';

class QuickSendVideo extends StatefulWidget {
  QuickSendVideo({this.video, this.contacts, this.leads});

  final Video video;
  final List<Contact> contacts;
  final List<Lead> leads;

  @override
  _QuickSendVideoState createState() => _QuickSendVideoState();
}

class _QuickSendVideoState extends State<QuickSendVideo> {
  int _currentStep = 0;
  final _controller = TextEditingController();
  final _subController = TextEditingController();
  final _bodyController = TextEditingController();
  final _scaffoldKey = GlobalKey<ScaffoldState>();
  List<Contact> _contacts;
  List<Lead> _leads;
  List<String> emails = [];
  List<api.LeadGroup> leadGroup;
  List<api.ContactGroup> contactGroup;
  String _query = "";
  String videoId;
  Video _video;

  void _clearSearch() {
    if (mounted) {
      setState(() {
        _controller.clear();
        _query = '';
      });
    }
  }

  double kGridKeight(BuildContext context) =>
      MediaQuery.of(context).size.height * 0.5;

  @override
  void initState() {
    super.initState();
    if (widget.contacts != null && widget.contacts.isNotEmpty) {
      _contacts = widget.contacts;
    }
    if (widget.leads != null && widget.leads.isNotEmpty) {
      _leads = widget.leads;
    }
    if (widget.video != null) {
      _video = widget.video;
    }
  }

  @override
  Widget build(BuildContext context) {
    final user = AuthBloc.currentInfo(context);
    return BlocBuilder<AuthBloc, AuthState>(
      builder: (context, state) {
        final _steps = [
          if (state is LoggedInState) ...[
            Step(
              title: Text('Select Video'),
              content: Card(
                child: Padding(
                  padding: EdgeInsets.all(8),
                  child: Wrap(
                    children: <Widget>[
                      Container(
                        child: TextField(
                          decoration: InputDecoration(
                              hintText: 'Search..',
                              border: InputBorder.none,
                              filled: true,
                              suffixIcon: IconButton(
                                icon: Icon(Icons.clear),
                                onPressed: _clearSearch,
                              )),
                          controller: _controller,
                          onChanged: (val) {
                            setState(() {
                              _query = val;
                            });
                          },
                        ),
                      ),
                      StreamBuilder<List<Video>>(
                        stream: GetIt.instance<DatabaseService>()
                            .videoDao
                            .watchVideosByUser(user.id),
                        builder: (context, snapshot) {
                          if (snapshot.hasData) {
                            if (snapshot.data.isEmpty) {
                              return Center(child: Text('No Videos Found'));
                            }
                            return Container(
                                height: kGridKeight(context),
                                child: ListView.builder(
                                  itemCount: snapshot.data.length,
                                  itemBuilder: (context, index) {
                                    final item = snapshot.data[index];
                                    return Padding(
                                      padding: EdgeInsets.all(8),
                                      child: ListTile(
                                        leading: Icon(Icons.videocam),
                                        title: Text(item.name),
                                        subtitle: Text(
                                          item.description,
                                          maxLines: 2,
                                        ),
                                        selected:
                                            _video?.videoId == item.videoId,
                                        onTap: () {
                                          if (mounted) {
                                            setState(() {
                                              _video = item;
                                            });
                                          }
                                        },
                                        trailing: Text('Views: ${item.views}'),
                                      ),
                                    );
                                  },
                                ));
                          }
                          return Container(
                            height: kGridKeight(context),
                            child: Center(
                              child: CircularProgressIndicator(),
                            ),
                          );
                        },
                      )
                    ],
                  ),
                ),
              ),
              state: _currentStep == 0
                  ? StepState.editing
                  : _video != null ? StepState.complete : StepState.indexed,
              isActive: true,
            ),
            Step(
                title: Text('Enter Email Content'),
                content: Card(
                  child: Padding(
                    padding: EdgeInsets.all(8),
                    child: Wrap(
                      children: <Widget>[
                        AppInputField(
                          controller: _subController,
                          multiLine: false,
                          name: 'Subject',
                        ),
                        AppInputField(
                          controller: _bodyController,
                          multiLine: true,
                          name: 'Body',
                        ),
                      ],
                    ),
                  ),
                ),
                isActive: true,
                state: _currentStep == 1
                    ? StepState.editing
                    : (_subController.text.isNotEmpty &&
                            _bodyController.text.isNotEmpty != null)
                        ? StepState.complete
                        : StepState.indexed),
            Step(
              title: Text('Select Contacts and Leads'),
              content: Card(
                child: Padding(
                  padding: EdgeInsets.all(8),
                  child: Column(
                    children: <Widget>[
                      Center(
                        child: ContactLeadEmailSelect(
                          contacts: _contacts,
                          leads: _leads,
                          contactsChanged: (items) {
                            if (mounted) {
                              setState(() {
                                _contacts = items;
                              });
                            }
                          },
                          leadsChanged: (items) {
                            if (mounted) {
                              setState(() {
                                _leads = items;
                              });
                            }
                          },
                          emailsChanged: (item) {
                            if (mounted) {
                              setState(() {
                                emails = item;
                              });
                            }
                          },
                          leadGroupSelected: (items) {
                            if (mounted) {
                              setState(() {
                                leadGroup = items;
                              });
                            }
                          },
                          contactGroupSelected: (items) {
                            if (mounted) {
                              setState(() {
                                contactGroup = items;
                              });
                            }
                          },
                        ),
                      ),
                    ],
                  ),
                ),
              ),
              state: _currentStep == 2
                  ? StepState.editing
                  : (_contacts != null ||
                          _leads != null ||
                          (emails != null && emails.isNotEmpty) ||
                          leadGroup != null ||
                          contactGroup != null)
                      ? StepState.complete
                      : StepState.indexed,
              isActive: true,
            ),
            Step(
              title: Text('Review and Send'),
              content: Card(
                child: _buildQuickReview(state),
              ),
            )
          ]
        ];
        return Scaffold(
          appBar: AppBar(
            title: Text('Quick Send Video'),
          ),
          body: Stepper(
            currentStep: _currentStep,
            onStepCancel: _onStepCancel,
            onStepTapped: _onStepTap,
            onStepContinue: state is LoggedInState
                ? () => _onStepContinue(_steps, user.client)
                : null,
            steps: _steps,
            type: StepperType.vertical,
          ),
        );
      },
    );
  }

  Widget _buildQuickReview(LoggedInState state) {
    getAllEmails();
    if (_video == null) {
      return Container(
        height: 100,
        child: Center(
          child: RaisedButton(
            child: Text('Pick a Video'),
            onPressed: () {
              if (mounted)
                setState(() {
                  _currentStep = 0;
                });
            },
          ),
        ),
      );
    }
    if ((_contacts == null || _contacts.isEmpty) &&
        (_leads == null || _leads.isEmpty) &&
        (leadGroup == null || leadGroup.isEmpty) &&
        (contactGroup == null || contactGroup.isEmpty) &&
        (emails == null)) {
      return Container(
        height: 100,
        child: Center(
          child: RaisedButton(
            child: Text('Pick Contacts/Leads'),
            onPressed: () {
              if (mounted)
                setState(() {
                  _currentStep = 2;
                });
            },
          ),
        ),
      );
    }
    if (_subController.text.isEmpty ||
        _subController.text == null ||
        _bodyController.text.isEmpty ||
        _bodyController.text == null) {
      return Container(
        height: 100,
        child: Center(
          child: RaisedButton(
            child: Text('Enter Subject and Body'),
            onPressed: () {
              if (mounted)
                setState(() {
                  _currentStep = 1;
                });
            },
          ),
        ),
      );
    }
    return Container(
      child: Center(
        child: Column(
          children: <Widget>[
            ListTile(
                leading: Icon(Icons.videocam),
                title: Text(_video.name),
                subtitle: Text(
                  _video.description,
                  maxLines: 2,
                ),
                trailing: Text('Views: ${_video.views}')),
            ListTile(
              title: Text('Subject:'),
              subtitle: Text(_subController.value.text),
            ),
            ListTile(
                title: Text('Body:'),
                subtitle: Text(_bodyController.value.text)),
            Divider(),
            ListTile(
              title: Text('Number of Recipients:'),
              trailing: Text(emails.length.toString(),
                  style: Theme.of(context).textTheme.display1),
            ),
          ],
        ),
      ),
    );
  }

  void getAllEmails() async {
    final client = AuthBloc.currentInfo(context).client;
    if (leadGroup != null && leadGroup.isNotEmpty) {
      for (var group in leadGroup) {
        final leads = await client.listEmailsForLeadGroup(group.id);
        emails.addAll(leads);
      }
    }
    if (contactGroup != null && contactGroup.isNotEmpty) {
      for (var group in contactGroup) {
        final contacts = await client.listEmailsForContactGroup(group.id);
        emails.addAll(contacts);
      }
    }
    if(_leads != null && _leads.isNotEmpty){
      for(var lead in _leads){
        if(!emails.contains(lead.email))
          emails.add(lead.email);
      }
    }
    if(_contacts != null && _contacts.isNotEmpty){
      for(var contact in _contacts){
        if(!emails.contains(contact.email))
          emails.add(contact.email);
      }
    }
  }

  void _onStepContinue(List<Step> _steps, api.UnifyClient client) async {
    if (_currentStep < _steps.length - 1) {
      setState(() {
        _currentStep += 1;
      });
    } else {
      try {
        await client.startVideoQuickSend(_video.videoId,
            emails: emails,
            subject: _subController.value.text,
            body: _bodyController.value.text,
            name: _video.name);
        _scaffoldKey.currentState.showSnackBar(SnackBar(
          content: Text('Video Sent!'),
        ));
        await Future.delayed(Duration(seconds: 3));
        Navigator.pop(context);
      } catch (e) {
        _scaffoldKey.currentState.showSnackBar(SnackBar(
          content: Text(e.toString()),
        ));
      }
    }

    print('onStepContinue : ' + _currentStep.toString());
  }

  void _onStepCancel() {
    setState(() {
      if (_currentStep > 0) {
        _currentStep -= 1;
      } else {
        _currentStep = 0;
      }
    });

    print('onStepCancel : ' + _currentStep.toString());
  }

  void _onStepTap(int step) {
    setState(() {
      _currentStep = step;
    });

    print('onStepTapped : ' + step.toString());
  }
}
